/*
 * Copyright (C) 2020 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.service.attestation;

import static com.android.internal.util.function.pooled.PooledLambda.obtainMessage;

import android.annotation.NonNull;
import android.annotation.Nullable;
import android.annotation.SystemApi;
import android.app.Service;
import android.content.Intent;
import android.graphics.Rect;
import android.hardware.HardwareBuffer;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.os.RemoteCallback;

/**
 * A service that handles generating and verify ImpressionTokens.
 *
 * The service will generate an ImpressionToken based on arguments passed in. Then later that same
 * ImpressionToken can be verified to determine that it was created by the system.
 *
 * @hide
 */
@SystemApi
public abstract class ImpressionAttestationService extends Service {
    /** @hide **/
    public static final String EXTRA_IMPRESSION_TOKEN =
            "android.service.attestation.extra.IMPRESSION_TOKEN";

    /** @hide **/
    public static final String EXTRA_VERIFICATION_STATUS =
            "android.service.attestation.extra.VERIFICATION_STATUS";

    /**
     * Manifest metadata key for the resource string array containing the names of all impression
     * attestation algorithms provided by the service.
     *
     * @hide
     */
    public static final String SERVICE_META_DATA_KEY_AVAILABLE_ALGORITHMS =
            "android.attestation.available_algorithms";

    /**
     * The {@link Intent} action that must be declared as handled by a service in its manifest
     * for the system to recognize it as an impression attestation providing service.
     *
     * @hide
     */
    public static final String SERVICE_INTERFACE =
            "android.service.attestation.ImpressionAttestationService";

    private ImpressionAttestationServiceWrapper mWrapper;
    private Handler mHandler;

    public ImpressionAttestationService() {
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mWrapper = new ImpressionAttestationServiceWrapper();
        mHandler = new Handler(Looper.getMainLooper(), null, true);
    }

    @NonNull
    @Override
    public final IBinder onBind(@NonNull Intent intent) {
        return mWrapper;
    }

    /**
     * Generates the impression token that can be used to validate that the system
     * generated the token.
     *
     * @param salt          The salt to use when generating the hmac. This should be unique to the
     *                      caller so the token cannot be verified by any other process.
     * @param screenshot    The screenshot buffer for the content to attest.
     * @param bounds        The size and position of the content being attested in the window.
     * @param hashAlgorithm The String for the hashing algorithm to use based values in
     *                      {@link #SERVICE_META_DATA_KEY_AVAILABLE_ALGORITHMS)}.
     * @return An impression token that can be used to validate information about the content.
     * Returns null when the arguments sent are invalid.
     */
    @Nullable
    public abstract ImpressionToken onGenerateImpressionToken(@NonNull String salt,
            @NonNull HardwareBuffer screenshot, @NonNull Rect bounds,
            @NonNull String hashAlgorithm);

    /**
     * Call to verify that the impressionToken passed in was generated by the system.
     *
     * @param salt            The salt value to use when verifying the hmac. This should be the
     *                        same value that was passed to
     *                        {@link #onGenerateImpressionToken(String,
     *                        HardwareBuffer, Rect, String)} to
     *                        generate the token.
     * @param impressionToken The token to verify that it was generated by the system.
     * @return true if the token can be verified that it was generated by the system.
     */
    public abstract boolean onVerifyImpressionToken(@NonNull String salt,
            @NonNull ImpressionToken impressionToken);

    private void generateImpressionToken(String salt, HardwareBuffer screenshot, Rect bounds,
            String hashAlgorithm, RemoteCallback callback) {
        ImpressionToken impressionToken = onGenerateImpressionToken(salt, screenshot, bounds,
                hashAlgorithm);
        final Bundle data = new Bundle();
        data.putParcelable(EXTRA_IMPRESSION_TOKEN, impressionToken);
        callback.sendResult(data);
    }

    private void verifyImpressionToken(String salt, ImpressionToken impressionToken,
            RemoteCallback callback) {
        boolean verificationStatus = onVerifyImpressionToken(salt, impressionToken);
        final Bundle data = new Bundle();
        data.putBoolean(EXTRA_VERIFICATION_STATUS, verificationStatus);
        callback.sendResult(data);
    }

    private final class ImpressionAttestationServiceWrapper extends
            IImpressionAttestationService.Stub {
        @Override
        public void generateImpressionToken(String salt, HardwareBuffer screenshot, Rect bounds,
                String hashAlgorithm, RemoteCallback callback) {
            mHandler.sendMessage(
                    obtainMessage(ImpressionAttestationService::generateImpressionToken,
                            ImpressionAttestationService.this, salt, screenshot, bounds,
                            hashAlgorithm, callback));
        }

        @Override
        public void verifyImpressionToken(String salt, ImpressionToken impressionToken,
                RemoteCallback callback) {
            mHandler.sendMessage(obtainMessage(ImpressionAttestationService::verifyImpressionToken,
                    ImpressionAttestationService.this, salt, impressionToken, callback));
        }
    }
}
